/************************************************************************/
/* ioloop.c							     */
/************************************************************************/
/*
 *  (c)2021,2022 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#include <stdint.h>
#include <poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
//#include <string.h>
#include <errno.h>
//#include <signal.h>
//#include <cutil/cutil.h>

#include <ioloop.h>


ioloop_t *ioloop_open(int options)
{
	ioloop_t *loop = NULL;
	loop = calloc(1, sizeof(ioloop_t));
	if (!loop)
		return NULL;
	loop->poll_timeout = IOLOOP_DEFAULT_TIMEOUT;
	return loop;
}


int ioloop_close(ioloop_t *loop)
{
	// TODO
	return 0;
}


int ioloop_set_default_io_callback(ioloop_t *loop, ioloop_callback_func func)
{
	loop->default_callback = func;
	return 0;
}


int ioloop_register_fd(ioloop_t *loop,
			int fd,
			uint8_t type,
			uint16_t port,
			ioloop_callback_func callback,
			void *callback_arg)
{
	if (loop->poll_nfds >= POLL_MAX_SOCKETS)
		return ENOSPC;
	loop->poll_fds[loop->poll_nfds].fd = fd;
	loop->poll_fds[loop->poll_nfds].events = POLLIN;
	loop->poll_conn_data[loop->poll_nfds].type = type;
	loop->poll_conn_data[loop->poll_nfds].port = port;
	loop->fd_callback[loop->poll_nfds] = callback;
	loop->fd_callback_arg[loop->poll_nfds] = callback_arg;
	loop->poll_nfds++;

	return 0;
}


int ioloop_unregister_fd(ioloop_t *loop, int fd)
{
	// TODO
	return 0;
}


int ioloop_run(ioloop_t *loop, int options, uint32_t timeout)
{
	int			rc;
	int			lfd;
	int			prc;
	ioloop_callback_func	cb;
	
	while (1) {
		prc = poll(loop->poll_fds, loop->poll_nfds, loop->poll_timeout);
		if (prc == -1) {
			if (errno != EINTR) break;
		}

		if (prc > 0) { 
			for (lfd=0 ; lfd<loop->poll_nfds; lfd++) {
				if (loop->poll_fds[lfd].revents & POLLIN) {
					if (loop->fd_callback[lfd] != NULL)
						cb = loop->fd_callback[lfd];
					else if (loop->default_callback != NULL)
						cb = loop->default_callback;
					if (cb) {
						rc = (*cb)(loop->poll_fds[lfd].fd,
							loop->poll_fds[lfd].revents,
							&loop->poll_conn_data[lfd],
							loop->fd_callback_arg[lfd]);
						(void)rc;
					}
				}
			}
		}
	}

	return 0;
}



